/*
**==============================================================================
**
** Open Management Infrastructure (OMI)
**
** Copyright (c) Microsoft Corporation
** 
** Licensed under the Apache License, Version 2.0 (the "License"); you may not 
** use this file except in compliance with the License. You may obtain a copy 
** of the License at 
**
**     http://www.apache.org/licenses/LICENSE-2.0 
**
** THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
** KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED 
** WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABLITY OR NON-INFRINGEMENT. 
**
** See the Apache 2 License for the specific language governing permissions 
** and limitations under the License.
**
**==============================================================================
*/

#ifndef _provmgr_filter_h
#define _provmgr_filter_h

#include <common.h>
#include <base/messages.h>
#include <wql/wql.h>

BEGIN_EXTERNC

typedef struct _IndicationFilter IndicationFilter;

/*
 * Defines common internal operations for structs derived from IndicationFilter.
 */
typedef struct _IndicationFilterFT
{
    /* Evaluates the specified indication using the pre-defined filter */
    MI_Result (*Filter)(
        _In_ IndicationFilter* self, 
        _In_ const MI_Instance* indication,
        _Out_ MI_Boolean* isMatch);

    /* Cleans up the IndicationFilter's internal state */
    void (*Destroy)(
        _In_ IndicationFilter* self);
}
IndicationFilterFT;

/*
 * Base structure for all "derived" IndicationFilters.
 */
struct _IndicationFilter
{
    MI_Filter base;
    MI_Uint32 magic;

    IndicationFilterFT* ft;

    SubscribeReq* msg;
};

/*
 * Generates a "derived" IndicationFilter structure based on the specified
 * query within the message.  It allocates the filter using the msg's
 * Batch and initializes internal values.
 *
 * Returns newly allocated filter or NULL for failure.
 */
IndicationFilter* IndicationFilter_New( 
    _In_ SubscribeReq* msg );

/*
 * Deactivates the filter and cleans it up.
 */
void IndicationFilter_Destroy(
    _Inout_ IndicationFilter* self );

MI_Result IndicationFilter_Filter(
    _In_ IndicationFilter* self, 
    _In_ const MI_Instance* indication,
    _Out_ MI_Boolean* isMatch );

END_EXTERNC

#endif /* _provmgr_filter_h */

